//
//  Copyright (c) 2004 Nat!
//  All rights reserved.
//
//  A Mulle kybernetiK production
//  www.mulle-kybernetik.com
//
#import "LEDImagePainter.h"


#define CACHE_ENABLED   0
#define MANUAL_W        9.0
#define BORDER_W        2
#define OFFSET_X        -1.5
#define OFFSET_Y        0.5


@implementation LEDImagePainter

- (id) initWithBundle:(NSBundle *) bundle
{
   NSString   *path;
   
   [super init];
      
   path = [bundle pathForResource:@"on-off"
                           ofType:@"tiff"];
   onOff_ = [[NSImage alloc] initByReferencingFile:path];
   
   path = [bundle pathForResource:@"on-on"
                           ofType:@"tiff"];
   onOn_ = [[NSImage alloc] initByReferencingFile:path];
   
   path = [bundle pathForResource:@"off-on"
                            ofType:@"tiff"];
   offOn_ = [[NSImage alloc] initByReferencingFile:path];
   
   path = [bundle pathForResource:@"off-off"
                             ofType:@"tiff"];
   offOff_ = [[NSImage alloc] initByReferencingFile:path];
   
   return( self);
}


- (void) dealloc
{
   free( array_);
   [image_ release];
   
   [super dealloc];
}


- (NSSize) imageSizeForBoolArrayLength:(unsigned int) n
{
   NSSize   size;
   
   size.height = [onOn_ size].height;
   size.width  = ((n + 1) / 2) * MANUAL_W + BORDER_W * 2;
   
   if( ! size.width)
      size.width = [onOn_ size].width;  // minimum size
   return( size);
}


- (NSImage *) _imageForBoolArray:(BOOL *) array
                          length:(unsigned int) n
{
   unsigned int  i;
   float         offset;
   int           flag1;
   int           flag2;
   NSRect        rect;
   NSPoint       dstXY;
   NSImage       *src;
   NSImage       *image;
   NSSize        size;
   
   size  = [self imageSizeForBoolArrayLength:n];
   image = [[[NSImage alloc] initWithSize:size] autorelease];
   
   [image lockFocus];
   
#if 0
   rect.origin = NSZeroPoint;
   rect.size   = size;
   
   NSFrameRect( rect);
#endif   
   dstXY  = NSMakePoint( OFFSET_X, OFFSET_Y);
   offset = size.width;
   
   for( i = 0; i < n; i += 2)
   {
      rect.origin = NSZeroPoint;
      rect.size   = [onOff_ size];
      
      flag2 = 0;
      flag1 = array[ i];
      if( i + 1 < n)
         flag2 = array[ i + 1];
      else
         rect.size.height /= 2;
      
      switch( (flag2 << 1) | flag1)
      {
      case 0 : src = offOff_; break;
      case 1 : src = onOff_;  break;
      case 2 : src = offOn_;  break;
      case 3 : src = onOn_;   break;
      }
      
      [src drawAtPoint:dstXY 
              fromRect:rect
             operation:NSCompositeSourceOver
              fraction:1.0];
      dstXY.x += MANUAL_W;
   }
   
   [image unlockFocus];
   
   return( image);
}


- (NSImage *) image
{
#if CACHE_ENABLED   
   if( ! image_)
      image_ = [[self _imageForBoolArray:array_
                                  length:length_] retain];
   return( image_);
#else
   return( [self _imageForBoolArray:array_
                             length:length_]);
#endif      
}


- (void) setBoolArray:(BOOL *) array
               length:(unsigned int) length
{
   size_t   bytes;
   
   NSParameterAssert( array);
   NSParameterAssert( length);
   
   bytes = sizeof( BOOL) * length;
   if( length == length_)
   {
      if( ! memcmp( array, array_, bytes))
         return;
   }
   
   free( array_);
   array_ = malloc( bytes);
   memcpy( array_, array, bytes);
   length_ = length;

#if 1
   {
      unsigned int   i;
      
      for( i = 0; i < length; i++)
         NSLog( @"%d = %d", i, array_[ i]);
   }
#endif      
   [image_ release];
   image_ = nil;
}

@end
