//
//  Copyright (c) 2004 Nat!
//  All rights reserved.
//
//  A Mulle kybernetiK production
//  www.mulle-kybernetik.com
//
#import "MulleHostCheck.h"
#import "HostPingList.h"


static NSString   *UserDefaultsHostListKey = @"com.mulle-kybernetik.software.MulleHostCheck.hostList";


@implementation MulleHostCheck

- (id) init
{
   [super init];
   
   refreshDelay_ = 11;  // 10 second ping time out...
   list_         = [[HostPingList alloc] init];
   lock_         = [[NSLock alloc] init];
   
   [lock_ lock];
   [self readUserDefaults];
   [lock_ unlock];

   // Register for pref changes
   [[NSDistributedNotificationCenter defaultCenter] addObserver:self 
                                                       selector:@selector( userDefaultsChanged:)
                                                           name:@"com.mulle-kybernetik.software.MulleHostCheck.preferencesChanged" 
                                                         object:nil];
   
   return( self);
}


- (void) _run
{
   NSAutoreleasePool   *pool;
   unsigned int        i;
   unsigned int        seconds;
   
   seconds  = refreshDelay_;
   running_ = YES;
   for(;;)
   {
      for( i = 0; i < seconds; i++)
      {
         if( stop_)
         {
            running_ = NO;
            return;
         }
         sleep( 1);
      }
      
      seconds = 1;
      if( [lock_ tryLock])
      {
      NS_DURING      
#if 1      
         pool = [[NSAutoreleasePool alloc] init];
         NSLog( @"update");
         [pool release];
#endif      
         [list_ updateAsynchronously];
      NS_HANDLER
      NS_ENDHANDLER   
         [lock_ unlock];
         seconds = refreshDelay_;
      }
   }
}


- (void) run
{
   [NSThread detachNewThreadSelector:@selector( _run)
                            toTarget:self
                          withObject:nil];
}


- (void) stop
{
   stop_ = YES;
   while( running_)
      sleep( 1);
}


- (void) shutdown
{
   [[NSDistributedNotificationCenter defaultCenter] removeObserver:self
                                                              name:@"com.mulle-kybernetik.software.MulleHostCheck.preferencesChanged" 
                                                            object:nil];
}


- (void) dealloc
{
   [self shutdown];
   [super dealloc];
}


- (void) readUserDefaults
{
   NSDictionary     *dictionary;
   NSArray          *array;
   NSHost           *host;
   NSUserDefaults   *defaults;
   HostPingList     *old;
   
   defaults   = [NSUserDefaults standardUserDefaults];
   dictionary = [defaults persistentDomainForName:@"com.apple.systempreferences"];
   array      = [dictionary objectForKey:UserDefaultsHostListKey];
      
   old   = list_;
   list_ = [[HostPingList alloc] init];

   if( array)
   {
      NSString       *s;
      NSEnumerator   *rover;
      NSDictionary   *item;
      
      NSAssert( [array isKindOfClass:[NSArray class]], @"not an array");
      
      rover = [array objectEnumerator];
      while( item = [rover nextObject])
      {
         NSAssert( [item isKindOfClass:[NSDictionary class]], @"not a dictionary");
  
         if( [[item objectForKey:@"enabled"] boolValue])
         {
            s = [item objectForKey:@"host"];
         
            host = [NSHost hostWithAddress:s];
            if( ! host)
               host = [NSHost hostWithName:s];
            if( host)
            {
               NSLog( @"Host %@", s);
               [list_ addHost:host];
            }
         }
      }
      NSLog( @"x");
   NS_DURING
      [list_ updateWithList:old];
   NS_HANDLER
      NSLog( [localException description]);
   NS_ENDHANDLER      
      NSLog( @"y");
   }      
   [old release];
}


- (void) userDefaultsChanged:(id) notification
{
   NSLog( @"Notification received");
   [[NSUserDefaults standardUserDefaults] synchronize];
   [self readUserDefaults];
}


- (unsigned int) count
{
   return( [list_ count]);
}


- (BOOL) isUpAtIndex:(unsigned int) index
{
   return( [list_ isUpAtIndex:index]);
}


- (NSString *) hostNameAtIndex:(unsigned int) index
{
   NSHost     *host;
   NSString   *s;
   
   host = [list_ hostAtIndex:index];
   s = [host name];
   if( ! s)
      s = [host address];
   return( s);
}

@end
